package swingtables.gui;

import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import swingtables.logic.Movie;
import swingtables.logic.MoviesDatasource;
import swingtables.logic.Genre;

/**
 *
 * @author trust56
 */
class MoviesTableModel extends AbstractTableModel implements TableModel {

    private Movie getMovieAtRow(int rowIndex) {
        return MoviesDatasource.getInstance().getMovies().get(rowIndex);
    }

    @Override
    public int getRowCount() {
        return MoviesDatasource.getInstance().getMovies().size();
    }

    @Override
    public int getColumnCount() {
        return 4;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Movie movie = getMovieAtRow(rowIndex);
        switch (columnIndex) {
            case 0:
                return movie.getTitle();
            case 1:
                return movie.getYearOfProduction();
            case 2:
                return movie.isColor();
            case 3:
                return movie.getGenre();
            default:
                return null;
        }
    }

    @Override
    public String getColumnName(int column) {
        return new String[]{"Title", "Year", "Color", "Genre"}[column];
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return String.class;
            case 1:
                return Integer.class;
            case 2:
                return Boolean.class;
            case 3:
                return Genre.class;
            default:
                return null;
        }
    }

    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Movie movie = getMovieAtRow(rowIndex);
        switch (columnIndex) {
            case 0:
                movie.setTitle(aValue.toString());
                break;
            case 1:
                try {
                    movie.setYearOfProduction(Integer.valueOf(aValue.toString()));
                } catch (IllegalArgumentException ex) {
                    JOptionPane.showMessageDialog(null, "Movie production year is too early", "Error", JOptionPane.ERROR_MESSAGE);
                }
                break;
            case 2:
                movie.setColor(Boolean.valueOf(aValue.toString()));
                break;
            case 3:
                movie.setGenre((Genre) aValue);
                break;
        }
    }

    public void addNewMovie() {
        int index = MoviesDatasource.getInstance().addMovie();
        fireTableRowsInserted(index, index);
    }

    public void deleteMovie(int rowIndex) {
        MoviesDatasource.getInstance().deleteMovie(rowIndex);
        fireTableRowsDeleted(rowIndex, rowIndex);
    }
}
